package com.persagy.ztkencryptdecodedata.kafka;

import com.alibaba.fastjson.JSONObject;
import com.persagy.ztkencryptdecodedata.dataSafety.EncryptInputMessageService;
import lombok.SneakyThrows;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.apache.kafka.common.KafkaException;
import org.apache.kafka.common.config.SslConfigs;
import org.apache.kafka.common.errors.AuthorizationException;
import org.apache.kafka.common.errors.OutOfOrderSequenceException;
import org.apache.kafka.common.errors.ProducerFencedException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Service;
import org.springframework.util.ResourceUtils;

import java.io.File;
import java.util.Properties;
import java.util.concurrent.*;

@Service
public class EdgeKafkaProducer  {

    private static Logger logger = LoggerFactory.getLogger(EdgeKafkaProducer.class.getClass());
    private static BlockingQueue messageQueue = null;

    private static Properties properties;
    private static KafkaProducer<String, String> producer;

    @Autowired
    private EncryptInputMessageService encryptInputMessageService;

    @Value("${spring.kafka.bootstrap-servers}")
    private String bootstrapServer;
    @Value("${spring.kafka.producer.acks}")
    private String ackSet;
    @Value("${spring.kafka.producer.retries}")
    private int retries;
    @Value("${spring.kafka.producer.batch-size}")
    private int batchSize;
    @Value("${spring.kafka.producer.buffer-memory}")
    private int bfferMemory;
    @Value("${spring.kafka.producer.client-id}")
    private String clientID;
    @Value("${spring.kafka.producer.key-serializer}")
    private String keySerializer;
    @Value("${spring.kafka.producer.value-serializer}")
    private String ValueSerializer;
    @Value("${spring.kafka.producer.security.protocol}")
    private  String securtyprotocol;
    @Value("${spring.kafka.producer.ssl.endpoint.identification.algorithm}")
    private  String algorithm;
    @Value("${spring.profiles.active}")
    private String active;
    @Value("${spring.location}")
    private String location;
    @Value("${spring.kafka.producer.ssl.trust-store-location}")
    private String truststorelocaltion;
    @Value("${spring.kafka.producer.ssl.key-store-password}")
    private String keystorepassword;

    @Value("${spring.kafka.consumer.ssl.key-store-location}")
    private String keytorelocaltion;
    @Value("${spring.kafka.producer.ssl.trust-store-password}")
    private String truststorepassword;
    @Value("${spring.kafka.producer.ssl.key-password}")
    private String keypassword;


   // private final  static  ExecutorService executorService = new ThreadPoolExecutor(10, 20, 60, TimeUnit.MILLISECONDS,new LinkedBlockingQueue<>());
   private final static ExecutorService executorService = new ThreadPoolExecutor(Runtime.getRuntime().availableProcessors(),Runtime.getRuntime().availableProcessors() * 2, 60, TimeUnit.SECONDS,new LinkedBlockingQueue(50000));
    private  int mm=0;
    @SneakyThrows
    public Future ProducerSend(String moudle, Object object) {

       // InputStream input=EdgeKafkaProducer.class.getClassLoader().getResourceAsStream("db.properties");
        properties = new Properties();
        // 连接的 kafka 集群地址
        properties.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, bootstrapServer);
        properties.put("acks", ackSet);//所有follower都响应了才认为消息提交成功，即"committed"
        properties.put("retries", retries);//retries = MAX 无限重试，直到你意识到出现了问题:)
        properties.put("batch.size", batchSize);//producer将试图批处理消息记录，以减少请求次数.默认的批量处理消息字节数
        properties.put("max.request.size", 1048576); //信息发送最大值1MB
        //batch.size当批量的数据大小达到设定值后，就会立即发送，不顾下面的linger.ms
        properties.put("retry.backoff.ms", 500);//设定重试时间间隔避免无效的频繁重试
        properties.put("client.id", location+clientID+active+"_"+String.valueOf(mm));//设定重试时间间隔避免无效的频繁重试
        properties.put("linger.ms", 5);//延迟1ms发送，这项设置将通过增加小的延迟来完成--即，不是立即发送一条记录，producer将会等待给定的延迟时间以允许其他消息记录发送，这些消息记录可以批量处理
        properties.put("buffer.memory", bfferMemory);//producer可以用来缓存数据的内存大小。
        properties.put("key.serializer", "org.apache.kafka.common.serialization.IntegerSerializer");
        properties.put("value.serializer", "org.apache.kafka.common.serialization.StringSerializer");
        properties.put("security.protocol", securtyprotocol);
        properties.put("ssl.endpoint.identification.algorithm",algorithm);
       // properties.put("ssl.truststore.location", ResourceUtils.getFile(truststorelocaltion).getPath());
       // properties.put("ssl.truststore.password", keystorepassword);ResourceUtils.getFile(keytorelocaltion).getPath()ResourceUtils.getFile(truststorelocaltion).getPath()
        if ("dev".equals(active)){

        }else{
            properties.put(SslConfigs.SSL_KEYSTORE_LOCATION_CONFIG,  new File(keytorelocaltion).getAbsolutePath());
            properties.put(SslConfigs.SSL_TRUSTSTORE_LOCATION_CONFIG, new File(truststorelocaltion).getAbsolutePath());
        }


        properties.put(SslConfigs.SSL_KEYSTORE_PASSWORD_CONFIG,keystorepassword);
        properties.put(SslConfigs.SSL_KEY_PASSWORD_CONFIG,keypassword);
        properties.put(SslConfigs.SSL_TRUSTSTORE_PASSWORD_CONFIG,truststorepassword);




        // producer = new KafkaProducer<String, String>(properties);
        producer = new KafkaProducer<String, String>(properties);
        String Topic = location + "_" + moudle;
        String message = object.toString();
        ProducerRecord<String, String> record = new ProducerRecord<>(Topic, message);//Topic Key Value
        // ProducerRecord<String, String> record = new ProducerRecord<>("BBF464_IoT_Edge_Test_1", message);//Topic Key Value
        Future future = producer.send(record);
        producer.close();
        return future;
    }


    public void setQueue(BlockingQueue<String> queue) {

        while (true) {
            messageQueue = queue;
            try {
                /// Thread.sleep(3000);
              //  Object messageObject = messageQueue.peek();
                Object messageObject = messageQueue.poll();
                if (null == messageObject) {
                    // System.out.println(">>>> queue has no data");
                    logger.info(">>> kafka queue size: " + messageQueue.size());
                    Thread.sleep(100);
                    //executorService.shutdown();

                    break;
                } else {

                    executorService.submit((Runnable) () -> {
                        mm++;
                        JSONObject object = JSONObject.parseObject(messageObject.toString());
                        try {

                            Future future = ProducerSend(object.getString("port") + "_" + object.getString("userid"), encryptInputMessageService.beforeBodyWrite(messageObject));
                            //   System.out.println("kafkafuture=="+future.isDone());
                            logger.info(">>> kafka status future: " + future.isDone());
                            //  System.out.println(">>> kafka queue size: " + messageQueue.size());
                            logger.info(">>> kafka queue size: " + messageQueue.size());
                            if (future.isDone()) {
                                logger.info(">>> Edge send  success msg to Cloud: " + messageObject.toString());
                                logger.info(">>> send success encryptmsg: " + encryptInputMessageService.beforeBodyWrite(messageObject).toString());
                                //  System.out.println("send success msg=== "+messageObject.toString());
                               // messageQueue.poll();
                            }
                            // System.out.println("-----------------");
                        } catch (ProducerFencedException | OutOfOrderSequenceException | AuthorizationException e) {
                            e.printStackTrace();
                            logger.error("--- kafka_send_error:" + e);
                        } catch (KafkaException e) {
                            // For all other exceptions, just abort the transaction and try again.
                            //  producer.abortTransaction();
                           // ProducerSend(object.getString("port") + "_" + object.getString("userid"), encryptInputMessageService.beforeBodyWrite(messageObject));
                            e.printStackTrace();
                            logger.error("--- kafka_client_error:" + e);
                        }
                        //  logger.info(">>> kafka message: " + messageObject.toString());
                        //Thread.sleep(100 );
                        // }

                    });


                }
            } catch (InterruptedException e) {
                logger.error("executorService_error" + e);
            } catch (Exception e) {
                e.printStackTrace();
                logger.error("executorService_error" + e);
            }
           // messageQueue.poll();
        }

      //  this.setQueue(messageQueue);
    }
}